@@ -33,16 +33,15 @@
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 #include <glib/gi18n.h>
-#include <dbus/dbus-glib.h>
 #include <gconf/gconf-client.h>
 #include <packagekit-glib/packagekit.h>
 #include <devkit-power-gobject/devicekit-power.h>
 
 #include "egg-debug.h"
 #include "egg-string.h"
-#include "egg-dbus-monitor.h"
 
 #include "gpk-common.h"
+#include "gpk-session.h"
 #include "gpk-auto-refresh.h"
 #include "gpk-enum.h"
 
@@ -52,10 +51,6 @@ static void     gpk_auto_refresh_finalize	(GObject            *object);
 #define GPK_AUTO_REFRESH_PERIODIC_CHECK		60*60	/* force check for updates every this much time */
 #define GPK_UPDATES_LOGIN_TIMEOUT		3	/* seconds */
 
-#define GS_DBUS_SERVICE				"org.gnome.ScreenSaver"
-#define GS_DBUS_PATH				"/org/gnome/ScreenSaver"
-#define GS_DBUS_INTERFACE			"org.gnome.ScreenSaver"
-
 /*
  * at startup, after a small delay, force a GetUpdates call
  * every hour (or any event) check:
@@ -71,11 +66,9 @@ struct GpkAutoRefreshPrivate
 	gboolean		 force_get_updates_login;
 	guint			 force_get_updates_login_timeout_id;
 	guint			 timeout_id;
-	EggDbusMonitor		*monitor_gs;
 	DkpClient		*client;
 	GConfClient		*gconf_client;
-	DBusGProxy		*proxy_gs;
-	DBusGConnection		*connection;
+	GpkSession		*session;
 	PkControl		*control;
 };
 
@@ -157,52 +150,24 @@ gpk_auto_refresh_signal_get_upgrades (GpkAutoRefresh *arefresh)
 }
 
 /**
- * gpk_auto_refresh_convert_frequency:
- *
- * Return value: The number of seconds for the frequency period,
- * or zero for never or no schema
+ * gpk_auto_refresh_get_frequency_prefs:
  **/
 static guint
-gpk_auto_refresh_convert_frequency (GpkFreqEnum freq)
+gpk_auto_refresh_get_frequency_prefs (GpkAutoRefresh *arefresh, const gchar *key)
 {
-	if (freq == GPK_FREQ_ENUM_UNKNOWN) {
-		egg_warning ("no schema");
-		return 0;
-	}
-	if (freq == GPK_FREQ_ENUM_NEVER)
-		return 0;
-	if (freq == GPK_FREQ_ENUM_HOURLY)
-		return 60*60;
-	if (freq == GPK_FREQ_ENUM_DAILY)
-		return 60*60*24;
-	if (freq == GPK_FREQ_ENUM_WEEKLY)
-		return 60*60*24*7;
-	egg_warning ("unknown frequency enum");
-	return 0;
-}
-
-/**
- * gpk_auto_refresh_convert_frequency_text:
- **/
-static guint
-gpk_auto_refresh_convert_frequency_text (GpkAutoRefresh *arefresh, const gchar *key)
-{
-	gchar *freq_text;
-	GpkFreqEnum freq;
+	guint value;
 
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), 0);
 
 	/* get from gconf */
-	freq_text = gconf_client_get_string (arefresh->priv->gconf_client, key, NULL);
-	if (freq_text == NULL) {
-		egg_warning ("no schema for %s", key);
-		return 0;
+	value = gconf_client_get_int (arefresh->priv->gconf_client, key, NULL);
+	if (value == 0) {
+		egg_warning ("no schema for %s, using daily", key);
+		value = 86400;
 	}
 
 	/* convert to enum and get seconds */
-	freq = gpk_freq_enum_from_text (freq_text);
-	g_free (freq_text);
-	return gpk_auto_refresh_convert_frequency (freq);
+	return value;
 }
 
 /**
@@ -218,7 +183,7 @@ gpk_auto_refresh_maybe_refresh_cache (GpkAutoRefresh *arefresh)
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
 	/* if we don't want to auto check for updates, don't do this either */
-	thresh = gpk_auto_refresh_convert_frequency_text (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
 	if (thresh == 0) {
 		egg_debug ("not when policy is to never refresh");
 		return FALSE;
@@ -237,7 +202,7 @@ gpk_auto_refresh_maybe_refresh_cache (GpkAutoRefresh *arefresh)
 	}
 
 	/* get this each time, as it may have changed behind out back */
-	thresh = gpk_auto_refresh_convert_frequency_text (arefresh, GPK_CONF_FREQUENCY_REFRESH_CACHE);
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_REFRESH_CACHE);
 	if (thresh == 0) {
 		egg_debug ("not when policy is to never refresh");
 		return FALSE;
@@ -283,7 +248,7 @@ gpk_auto_refresh_maybe_get_updates (GpkAutoRefresh *arefresh)
 	}
 
 	/* get this each time, as it may have changed behind out back */
-	thresh = gpk_auto_refresh_convert_frequency_text (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_GET_UPDATES);
 	if (thresh == 0) {
 		egg_debug ("not when policy is set to never get updates");
 		return FALSE;
@@ -320,7 +285,7 @@ gpk_auto_refresh_maybe_get_upgrades (GpkAutoRefresh *arefresh)
 	g_return_val_if_fail (GPK_IS_AUTO_REFRESH (arefresh), FALSE);
 
 	/* get this each time, as it may have changed behind out back */
-	thresh = gpk_auto_refresh_convert_frequency_text (arefresh, GPK_CONF_FREQUENCY_GET_UPGRADES);
+	thresh = gpk_auto_refresh_get_frequency_prefs (arefresh, GPK_CONF_FREQUENCY_GET_UPGRADES);
 	if (thresh == 0) {
 		egg_debug ("not when policy is set to never check for upgrades");
 		return FALSE;
@@ -417,21 +382,21 @@ static void
 gpk_auto_refresh_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, GpkAutoRefresh *arefresh)
 {
 	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
-	if (egg_strequal (entry->key, GPK_CONF_SESSION_STARTUP_TIMEOUT) ||
-	    egg_strequal (entry->key, GPK_CONF_FORCE_GET_UPDATES_LOGIN) ||
-	    egg_strequal (entry->key, GPK_CONF_FREQUENCY_GET_UPDATES) ||
-	    egg_strequal (entry->key, GPK_CONF_FREQUENCY_GET_UPGRADES) ||
-	    egg_strequal (entry->key, GPK_CONF_FREQUENCY_REFRESH_CACHE) ||
-	    egg_strequal (entry->key, GPK_CONF_AUTO_UPDATE) ||
-	    egg_strequal (entry->key, GPK_CONF_UPDATE_BATTERY))
+	if (g_strcmp0 (entry->key, GPK_CONF_SESSION_STARTUP_TIMEOUT) == 0 ||
+	    g_strcmp0 (entry->key, GPK_CONF_FORCE_GET_UPDATES_LOGIN) == 0 ||
+	    g_strcmp0 (entry->key, GPK_CONF_FREQUENCY_GET_UPDATES) == 0 ||
+	    g_strcmp0 (entry->key, GPK_CONF_FREQUENCY_GET_UPGRADES) == 0 ||
+	    g_strcmp0 (entry->key, GPK_CONF_FREQUENCY_REFRESH_CACHE) == 0 ||
+	    g_strcmp0 (entry->key, GPK_CONF_AUTO_UPDATE) == 0 ||
+	    g_strcmp0 (entry->key, GPK_CONF_UPDATE_BATTERY) == 0)
 		gpk_auto_refresh_change_state (arefresh);
 }
 
 /**
- * gpk_auto_refresh_idle_cb:
+ * gpk_auto_refresh_session_idle_changed_cb:
  **/
 static void
-gpk_auto_refresh_idle_cb (DBusGProxy *proxy, gboolean is_idle, GpkAutoRefresh *arefresh)
+gpk_auto_refresh_session_idle_changed_cb (GpkSession *session, gboolean is_idle, GpkAutoRefresh *arefresh)
 {
 	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
 
@@ -513,44 +478,6 @@ gpk_auto_refresh_timeout_cb (gpointer user_data)
 	return TRUE;
 }
 
-/**
- * pk_connection_gs_changed_cb:
- **/
-static void
-pk_connection_gs_changed_cb (EggDbusMonitor *egg_dbus_monitor, gboolean connected, GpkAutoRefresh *arefresh)
-{
-	GError *error = NULL;
-
-	g_return_if_fail (GPK_IS_AUTO_REFRESH (arefresh));
-
-	egg_debug ("gnome-screensaver connection-changed: %i", connected);
-
-	/* is this valid? */
-	if (!connected) {
-		if (arefresh->priv->proxy_gs != NULL) {
-			g_object_unref (arefresh->priv->proxy_gs);
-			arefresh->priv->proxy_gs = NULL;
-		}
-		return;
-	}
-
-	/* use gnome-screensaver for the idle detection */
-	arefresh->priv->proxy_gs = dbus_g_proxy_new_for_name_owner (arefresh->priv->connection,
-					  GS_DBUS_SERVICE, GS_DBUS_PATH, GS_DBUS_INTERFACE, &error);
-	if (error != NULL) {
-		egg_warning ("Cannot connect to gnome-screensaver: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-	/* get SessionIdleChanged */
-	dbus_g_proxy_add_signal (arefresh->priv->proxy_gs, "SessionIdleChanged",
-				 G_TYPE_BOOLEAN, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (arefresh->priv->proxy_gs, "SessionIdleChanged",
-				     G_CALLBACK (gpk_auto_refresh_idle_cb),
-				     arefresh, NULL);
-
-}
-
 /**
  * gpk_auto_refresh_client_changed_cb:
  **/
@@ -582,17 +509,14 @@ gpk_auto_refresh_client_changed_cb (DkpClient *client, GpkAutoRefresh *arefresh)
 static void
 gpk_auto_refresh_init (GpkAutoRefresh *arefresh)
 {
-	GError *error = NULL;
 	PkNetworkEnum state;
 
 	arefresh->priv = GPK_AUTO_REFRESH_GET_PRIVATE (arefresh);
 	arefresh->priv->on_battery = FALSE;
-	arefresh->priv->session_idle = FALSE;
 	arefresh->priv->network_active = FALSE;
 	arefresh->priv->force_get_updates_login = FALSE;
 	arefresh->priv->timeout_id = 0;
 	arefresh->priv->force_get_updates_login_timeout_id = 0;
-	arefresh->priv->proxy_gs = NULL;
 
 	/* we need to know the updates frequency */
 	arefresh->priv->gconf_client = gconf_client_get_default ();
@@ -611,14 +535,6 @@ gpk_auto_refresh_init (GpkAutoRefresh *arefresh)
 	state = pk_control_get_network_state (arefresh->priv->control, NULL);
 	arefresh->priv->network_active = gpk_auto_refresh_convert_network_state (arefresh, state);
 
-	/* connect to session bus */
-	arefresh->priv->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-	if (error != NULL) {
-		egg_warning ("Cannot connect to session bus: %s", error->message);
-		g_error_free (error);
-		return;
-	}
-
 	/* use a DkpClient */
 	arefresh->priv->client = dkp_client_new ();
 	g_signal_connect (arefresh->priv->client, "changed",
@@ -628,11 +544,11 @@ gpk_auto_refresh_init (GpkAutoRefresh *arefresh)
 	arefresh->priv->on_battery = dkp_client_on_battery (arefresh->priv->client);
 	egg_debug ("setting on battery %i", arefresh->priv->on_battery);
 
-	/* watch gnome-screensaver */
-	arefresh->priv->monitor_gs = egg_dbus_monitor_new ();
-	g_signal_connect (arefresh->priv->monitor_gs, "connection-changed",
-			  G_CALLBACK (pk_connection_gs_changed_cb), arefresh);
-	egg_dbus_monitor_assign (arefresh->priv->monitor_gs, EGG_DBUS_MONITOR_SESSION, GS_DBUS_SERVICE);
+	/* use gnome-session for the idle detection */
+	arefresh->priv->session = gpk_session_new ();
+	g_signal_connect (arefresh->priv->session, "idle_changed",
+			  G_CALLBACK (gpk_auto_refresh_session_idle_changed_cb), arefresh);
+	arefresh->priv->session_idle = gpk_session_get_idle (arefresh->priv->session);
 
 	/* we check this in case we miss one of the async signals */
 	g_timeout_add_seconds (GPK_AUTO_REFRESH_PERIODIC_CHECK, gpk_auto_refresh_timeout_cb, arefresh);
@@ -661,13 +577,9 @@ gpk_auto_refresh_finalize (GObject *object)
 		g_source_remove (arefresh->priv->force_get_updates_login_timeout_id);
 
 	g_object_unref (arefresh->priv->control);
-	g_object_unref (arefresh->priv->monitor_gs);
 	g_object_unref (arefresh->priv->gconf_client);
 	g_object_unref (arefresh->priv->client);
-
-	/* only unref the proxies if they were ever set */
-	if (arefresh->priv->proxy_gs != NULL)
-		g_object_unref (arefresh->priv->proxy_gs);
+	g_object_unref (arefresh->priv->session);
 
 	G_OBJECT_CLASS (gpk_auto_refresh_parent_class)->finalize (object);
 }
